/*
 *  osconf.h - ACPI OS-dependent headers for Kernel Boot/Configuration time
 *
 *  Copyright (C) 2000 Intel Corp.
 *  Copyright (C) 2000 J.I. Lee <Jung-Ik.Lee@intel.com>
 */


struct acpi_osd {
	void * (*allocate)(u32 size);
	void * (*callocate)(u32 size);
	void  (*free)(void *ptr);
	ACPI_STATUS (*queue_for_exec)(u32 pri, OSD_EXECUTION_CALLBACK cb, void *context);
	ACPI_STATUS (*read_pci_cfg_byte)(u32 bus, u32 func, u32 addr, u8 *val);
	ACPI_STATUS (*read_pci_cfg_word)(u32 bus, u32 func, u32 addr, u16 *val);
	ACPI_STATUS (*read_pci_cfg_dword)(u32 bus, u32 func, u32 addr, u32 *val);
	ACPI_STATUS (*write_pci_cfg_byte)(u32 bus, u32 func, u32 addr, u8 val);
	ACPI_STATUS (*write_pci_cfg_word)(u32 bus, u32 func, u32 addr, u16 val);
	ACPI_STATUS (*write_pci_cfg_dword)(u32 bus, u32 func, u32 addr, u32 val);
};


#define	PCI_CONFIG_ADDRESS(bus, devfn, where) \
	(((u64) bus << 16) | ((u64) (devfn & 0xff) << 8) | (where & 0xff))

#define	ACPI_CF_PHASE_BOOTTIME	0x00
#define	ACPI_CF_PHASE_RUNTIME	0x01


/* acpi_osd functions */
void * acpi_os_allocate(u32 size);
void * acpi_os_callocate(u32 size);
void acpi_os_free(void *ptr);


ACPI_STATUS
acpi_os_read_pci_cfg_byte( u32 segbus, u32 func, u32 addr, u8 * val);

ACPI_STATUS
acpi_os_read_pci_cfg_word( u32 segbus, u32 func, u32 addr, u16 * val);

ACPI_STATUS
acpi_os_read_pci_cfg_dword( u32 segbus, u32 func, u32 addr, u32 * val);

ACPI_STATUS
acpi_os_write_pci_cfg_byte( u32 segbus, u32 func, u32 addr, u8 val);

ACPI_STATUS
acpi_os_write_pci_cfg_word( u32 segbus, u32 func, u32 addr, u16 val);

ACPI_STATUS
acpi_os_write_pci_cfg_dword( u32 segbus, u32 func, u32 addr, u32 val);


/* acpi_osd_rt functions */
extern void * acpi_os_allocate_rt(u32 size);
extern void * acpi_os_callocate_rt(u32 size);
extern void acpi_os_free_rt(void *ptr);

extern ACPI_STATUS
acpi_os_queue_for_execution_rt(
	u32			priority,
	OSD_EXECUTION_CALLBACK	callback,
	void			*context
	);

extern ACPI_STATUS
acpi_os_read_pci_cfg_byte_rt( u32 segbus, u32 func, u32 addr, u8 * val);

extern ACPI_STATUS
acpi_os_read_pci_cfg_word_rt( u32 segbus, u32 func, u32 addr, u16 * val);

extern ACPI_STATUS
acpi_os_read_pci_cfg_dword_rt( u32 segbus, u32 func, u32 addr, u32 * val);

extern ACPI_STATUS
acpi_os_write_pci_cfg_byte_rt( u32 segbus, u32 func, u32 addr, u8 val);

extern ACPI_STATUS
acpi_os_write_pci_cfg_word_rt( u32 segbus, u32 func, u32 addr, u16 val);

extern ACPI_STATUS
acpi_os_write_pci_cfg_dword_rt( u32 segbus, u32 func, u32 addr, u32 val);

